@@ -35,7 +35,8 @@ module Excon
     #   @param [Hash<Symbol, >] params One or more optional params
     #     @option params [String] :body Default text to be sent over a socket. Only used if :body absent in Connection#request params
     #     @option params [Hash<Symbol, String>] :headers The default headers to supply in a request. Only used if params[:headers] is not supplied to Connection#request
-    #     @option params [String] :host The destination host's reachable DNS name or IP, in the form of a String
+    #     @option params [String] :host The destination host's reachable DNS name or IP, in the form of a String. IPv6 addresses must be wrapped (e.g. [::1]).  See URI#host.
+    #     @option params [String] :hostname Same as host, but usable for socket connections. IPv6 addresses must not be wrapped (e.g. ::1).  See URI#hostname.
     #     @option params [String] :path Default path; appears after 'scheme://host:port/'. Only used if params[:path] is not supplied to Connection#request
     #     @option params [Fixnum] :port The port on which to connect, to the destination host
     #     @option params [Hash]   :query Default query; appended to the 'scheme://host:port/path/' in the form of '?key=value'. Will only be used if params[:query] is not supplied to Connection#request
@@ -57,36 +58,15 @@ module Excon
       params = validate_params(:connection, params)
       @data.merge!(params)
 
-      unless @data[:scheme] == UNIX
-        no_proxy_env = ENV["no_proxy"] || ENV["NO_PROXY"] || ""
-        no_proxy_list = no_proxy_env.scan(/\*?\.?([^\s,:]+)(?::(\d+))?/i).map { |s| [s[0], s[1]] }
-        unless no_proxy_list.index { |h| /(^|\.)#{h[0]}$/.match(@data[:host]) && (h[1].nil? || h[1].to_i == @data[:port]) }
-          if @data[:scheme] == HTTPS && (ENV.has_key?('https_proxy') || ENV.has_key?('HTTPS_PROXY'))
-            @data[:proxy] = setup_proxy(ENV['https_proxy'] || ENV['HTTPS_PROXY'])
-          elsif (ENV.has_key?('http_proxy') || ENV.has_key?('HTTP_PROXY'))
-            @data[:proxy] = setup_proxy(ENV['http_proxy'] || ENV['HTTP_PROXY'])
-          elsif @data.has_key?(:proxy)
-            @data[:proxy] = setup_proxy(@data[:proxy])
-          end
-        end
-      end
+      setup_proxy
 
-      if @data[:proxy]
-        @data[:headers]['Proxy-Connection'] ||= 'Keep-Alive'
-        # https credentials happen in handshake
-        if @data[:scheme] == 'http' && (@data[:proxy][:user] || @data[:proxy][:password])
-          user, pass = Utils.unescape_form(@data[:proxy][:user].to_s), Utils.unescape_form(@data[:proxy][:password].to_s)
-          auth = ['' << user.to_s << ':' << pass.to_s].pack('m').delete(Excon::CR_NL)
-          @data[:headers]['Proxy-Authorization'] = 'Basic ' << auth
-        end
-      end
-
-      if ENV.has_key?('EXCON_DEBUG') || ENV.has_key?('EXCON_STANDARD_INSTRUMENTOR')
+      if  ENV.has_key?('EXCON_STANDARD_INSTRUMENTOR')
         @data[:instrumentor] = Excon::StandardInstrumentor
+      end
 
-        if ENV.has_key?('EXCON_DEBUG')
-          @data[:debug_request] = @data[:debug_reponse] = true
-        end
+      if @data[:debug] || ENV.has_key?('EXCON_DEBUG')
+        @data[:debug_request] = @data[:debug_response] = true
+        @data[:instrumentor] = Excon::StandardInstrumentor
       end
 
       # Use Basic Auth if url contains a login
@@ -126,7 +106,7 @@ module Excon
           socket.data = datum
           # start with "METHOD /path"
           request = datum[:method].to_s.upcase << ' '
-          if datum[:proxy]
+          if datum[:proxy] && datum[:scheme] != HTTPS
             request << datum[:scheme] << '://' << datum[:host] << port_string(datum)
           end
           request << datum[:path]
@@ -151,13 +131,6 @@ module Excon
             end
           end
 
-          if datum[:response_block]
-            datum[:headers]['TE'] = 'trailers'
-          else
-            datum[:headers]['TE'] = 'trailers, deflate, gzip'
-          end
-          datum[:headers]['Connection'] = datum[:persistent] ? 'TE' : 'TE, close'
-
           # add headers to request
           datum[:headers].each do |key, values|
             [values].flatten.each do |value|
@@ -186,8 +159,8 @@ module Excon
             if body.respond_to?(:binmode)
               body.binmode
             end
-            if body.respond_to?(:pos=)
-              body.pos = 0
+            if body.respond_to?(:rewind)
+              body.rewind  rescue nil
             end
             while chunk = body.read(datum[:chunk_size])
               socket.write(chunk)
@@ -207,8 +180,10 @@ module Excon
     end
 
     def response_call(datum)
+      # ensure response_block is yielded to and body is empty from middlewares
       if datum.has_key?(:response_block) && !datum[:response][:body].empty?
         response_body = datum[:response][:body].dup
+        datum[:response][:body] = ''
         content_length = remaining = response_body.bytesize
         while remaining > 0
           datum[:response_block].call(response_body.slice!(0, [datum[:chunk_size], remaining].min), [remaining - datum[:chunk_size], 0].max, content_length)
@@ -248,17 +223,6 @@ module Excon
         datum[:response_block] = Proc.new
       end
 
-      if datum[:idempotent]
-        if datum[:request_block]
-          Excon.display_warning('Excon requests with a :request_block can not be :idempotent.')
-          datum[:idempotent] = false
-        end
-        if datum[:pipeline]
-          Excon.display_warning("Excon requests can not be :idempotent when pipelining.")
-          datum[:idempotent] = false
-        end
-      end
-
       datum[:connection] = self
 
       datum[:stack] = datum[:middlewares].map do |middleware|
@@ -273,7 +237,7 @@ module Excon
 
         if datum[:persistent]
           if key = datum[:response][:headers].keys.detect {|k| k.casecmp('Connection') == 0 }
-            if split_header_value(datum[:response][:headers][key]).any? {|t| t.casecmp('close') == 0 }
+            if datum[:response][:headers][key].casecmp('close') == 0
               reset
             end
           end
@@ -309,7 +273,7 @@ module Excon
 
       if @data[:persistent]
         if key = responses.last[:headers].keys.detect {|k| k.casecmp('Connection') == 0 }
-          if split_header_value(responses.last[:headers][key]).any? {|t| t.casecmp('close') == 0 }
+          if responses.last[:headers][key].casecmp('close') == 0
             reset
           end
         end
@@ -384,9 +348,9 @@ module Excon
     def validate_params(validation, params)
       valid_keys = case validation
       when :connection
-        valid_connection_keys(params)
+        Excon::VALID_CONNECTION_KEYS
       when :request
-        valid_request_keys(params)
+        Excon::VALID_REQUEST_KEYS
       end
       invalid_keys = params.keys - valid_keys
       unless invalid_keys.empty?
@@ -395,6 +359,12 @@ module Excon
         #params = params.dup
         #invalid_keys.each {|key| params.delete(key) }
       end
+
+      if validation == :connection && params.key?(:host) && !params.key?(:hostname)
+        Excon.display_warning('hostname is missing! For IPv6 support, provide both host and hostname: Excon::Connection#new(:host => uri.host, :hostname => uri.hostname, ...).')
+        params[:hostname] = params[:host]
+      end
+
       params
     end
 
@@ -410,35 +380,89 @@ module Excon
     end
 
     def socket
-      sockets[@socket_key] ||= if @data[:scheme] == HTTPS
-        Excon::SSLSocket.new(@data)
-      elsif @data[:scheme] == UNIX
+      unix_proxy = @data[:proxy] ? @data[:proxy][:scheme] == UNIX : false
+      sockets[@socket_key] ||= if @data[:scheme] == UNIX || unix_proxy
         Excon::UnixSocket.new(@data)
+      elsif @data[:scheme] == HTTPS
+        Excon::SSLSocket.new(@data)
       else
         Excon::Socket.new(@data)
       end
     end
 
     def sockets
-      Thread.current[:_excon_sockets] ||= {}
+      if @data[:thread_safe_sockets]
+        Thread.current[:_excon_sockets] ||= {}
+      else
+        @_excon_sockets ||= {}
+      end
     end
 
-    def setup_proxy(proxy)
-      case proxy
-      when String
-        uri = URI.parse(proxy)
-        unless uri.host and uri.port and uri.scheme
+    def setup_proxy
+      if @data[:disable_proxy]
+        if @data[:proxy]
+          raise ArgumentError, "`:disable_proxy` parameter and `:proxy` parameter cannot both be set at the same time."
+        end
+        return
+      end
+
+      unless @data[:scheme] == UNIX
+        if no_proxy_env = ENV["no_proxy"] || ENV["NO_PROXY"]
+          no_proxy_list = no_proxy_env.scan(/\*?\.?([^\s,:]+)(?::(\d+))?/i).map { |s| [s[0], s[1]] }
+        end
+
+        unless no_proxy_env && no_proxy_list.index { |h| /(^|\.)#{h[0]}$/.match(@data[:host]) && (h[1].nil? || h[1].to_i == @data[:port]) }
+          if @data[:scheme] == HTTPS && (ENV.has_key?('https_proxy') || ENV.has_key?('HTTPS_PROXY'))
+            @data[:proxy] = ENV['https_proxy'] || ENV['HTTPS_PROXY']
+          elsif (ENV.has_key?('http_proxy') || ENV.has_key?('HTTP_PROXY'))
+            @data[:proxy] = ENV['http_proxy'] || ENV['HTTP_PROXY']
+          end
+        end
+
+        case @data[:proxy]
+        when nil
+          @data.delete(:proxy)
+        when Hash
+          # no processing needed
+        when String, URI
+          uri = @data[:proxy].is_a?(String) ? URI.parse(@data[:proxy]) : @data[:proxy]
+          @data[:proxy] = {
+            :host       => uri.host,
+            :hostname   => uri.hostname,
+            # path is only sensible for a Unix socket proxy
+            :path       => uri.scheme == UNIX ? uri.path : nil,
+            :port       => uri.port,
+            :scheme     => uri.scheme,
+          }
+          if uri.password
+            @data[:proxy][:password] = uri.password
+          end
+          if uri.user
+            @data[:proxy][:user] = uri.user
+          end
+          if @data[:proxy][:scheme] == UNIX
+            if @data[:proxy][:host]
+              raise ArgumentError, "The `:host` parameter should not be set for `unix://` proxies.\n" +
+                                   "When supplying a `unix://` URI, it should start with `unix:/` or `unix:///`."
+            end
+          else
+            unless uri.host && uri.port && uri.scheme
+              raise Excon::Errors::ProxyParseError, "Proxy is invalid"
+            end
+          end
+        else
           raise Excon::Errors::ProxyParseError, "Proxy is invalid"
         end
-        {
-          :host       => uri.host,
-          :password   => uri.password,
-          :port       => uri.port,
-          :scheme     => uri.scheme,
-          :user       => uri.user
-        }
-      else
-        proxy
+
+        if @data.has_key?(:proxy) && @data[:scheme] == 'http'
+          @data[:headers]['Proxy-Connection'] ||= 'Keep-Alive'
+          # https credentials happen in handshake
+          if @data[:proxy].has_key?(:user) || @data[:proxy].has_key?(:password)
+            user, pass = Utils.unescape_form(@data[:proxy][:user].to_s), Utils.unescape_form(@data[:proxy][:password].to_s)
+            auth = ['' << user << ':' << pass].pack('m').delete(Excon::CR_NL)
+            @data[:headers]['Proxy-Authorization'] = 'Basic ' << auth
+          end
+        end
       end
     end
   end
