@@ -22,6 +22,12 @@ module Excon
     def status
       @data[:status]
     end
+    def reason_phrase=(new_reason_phrase)
+      @data[:reason_phrase] = new_reason_phrase
+    end
+    def reason_phrase
+      @data[:reason_phrase]
+    end
     def remote_ip=(new_remote_ip)
       @data[:remote_ip] = new_remote_ip
     end
@@ -37,18 +43,28 @@ module Excon
 
     def self.parse(socket, datum)
       # this will discard any trailing lines from the previous response if any.
-      until match = /^HTTP\/\d+\.\d+\s(\d{3})\s/.match(socket.readline); end
-      status = match[1].to_i
+      begin
+        line = socket.readline
+      end until status = line[9, 3].to_i
+
+      reason_phrase = line[13..-3] # -3 strips the trailing "\r\n"
 
       datum[:response] = {
         :body          => '',
         :headers       => Excon::Headers.new,
         :status        => status,
-        :remote_ip     => socket.respond_to?(:remote_ip) && socket.remote_ip,
-        :local_port    => socket.respond_to?(:local_port) && socket.local_port,
-        :local_address => socket.respond_to?(:local_address) && socket.local_address
+        :reason_phrase => reason_phrase
       }
 
+      unix_proxy = datum[:proxy] ? datum[:proxy][:scheme] == UNIX : false
+      unless datum[:scheme] == UNIX || unix_proxy
+        datum[:response].merge!(
+          :remote_ip     => socket.remote_ip,
+          :local_port    => socket.local_port,
+          :local_address => socket.local_address
+        )
+      end
+
       parse_headers(socket, datum)
 
       unless (['HEAD', 'CONNECT'].include?(datum[:method].to_s.upcase)) || NO_ENTITY.include?(datum[:response][:status])
@@ -57,13 +73,11 @@ module Excon
           encodings = Utils.split_header_value(datum[:response][:headers][key])
           if (encoding = encodings.last) && encoding.casecmp('chunked') == 0
             transfer_encoding_chunked = true
-            encodings.pop
-            datum[:response][:headers][key] = encodings.join(', ')
-          end
-        end
-        unless transfer_encoding_chunked
-          if key = datum[:response][:headers].keys.detect {|k| k.casecmp('Content-Length') == 0 }
-            content_length = datum[:response][:headers][key].to_i
+            if encodings.length == 1
+              datum[:response][:headers].delete(key)
+            else
+              datum[:response][:headers][key] = encodings[0...-1].join(', ')
+            end
           end
         end
 
@@ -76,36 +90,61 @@ module Excon
         end
 
         if transfer_encoding_chunked
-          # 2 == "\r\n".length
           if response_block
-            while (chunk_size = socket.readline.chop!.to_i(16)) > 0
-              response_block.call(socket.read(chunk_size + 2).chop!, nil, nil)
+            while (chunk_size = socket.readline.chomp!.to_i(16)) > 0
+              while chunk_size > 0
+                chunk = socket.read(chunk_size)
+                chunk_size -= chunk.bytesize
+                response_block.call(chunk, nil, nil)
+              end
+              new_line_size = 2 # 2 == "\r\n".length
+              while new_line_size > 0
+                new_line_size -= socket.read(new_line_size).length
+              end
             end
           else
-            while (chunk_size = socket.readline.chop!.to_i(16)) > 0
-              datum[:response][:body] << socket.read(chunk_size + 2).chop!
+            while (chunk_size = socket.readline.chomp!.to_i(16)) > 0
+              while chunk_size > 0
+                chunk = socket.read(chunk_size)
+                chunk_size -= chunk.bytesize
+                datum[:response][:body] << chunk
+              end
+              new_line_size = 2 # 2 == "\r\n".length
+              while new_line_size > 0
+                new_line_size -= socket.read(new_line_size).length
+              end
             end
           end
           parse_headers(socket, datum) # merge trailers into headers
-        elsif remaining = content_length
-          if response_block
-            while remaining > 0
-              response_block.call(socket.read([datum[:chunk_size], remaining].min), [remaining - datum[:chunk_size], 0].max, content_length)
-              remaining -= datum[:chunk_size]
-            end
-          else
-            while remaining > 0
-              datum[:response][:body] << socket.read([datum[:chunk_size], remaining].min)
-              remaining -= datum[:chunk_size]
-            end
-          end
         else
-          if response_block
-            while chunk = socket.read(datum[:chunk_size])
-              response_block.call(chunk, nil, nil)
+          if key = datum[:response][:headers].keys.detect {|k| k.casecmp('Content-Length') == 0 }
+            content_length = datum[:response][:headers][key].to_i
+          end
+
+          if remaining = content_length
+            if response_block
+              while remaining > 0
+                chunk = socket.read([datum[:chunk_size], remaining].min)
+                response_block.call(chunk, [remaining - chunk.bytesize, 0].max, content_length)
+                remaining -= chunk.bytesize
+              end
+            else
+              while remaining > 0
+                chunk = socket.read([datum[:chunk_size], remaining].min)
+                datum[:response][:body] << chunk
+                remaining -= chunk.bytesize
+              end
             end
           else
-            datum[:response][:body] << socket.read
+            if response_block
+              while chunk = socket.read(datum[:chunk_size])
+                response_block.call(chunk, nil, nil)
+              end
+            else
+              while chunk = socket.read(datum[:chunk_size])
+                datum[:response][:body] << chunk
+              end
+            end
           end
         end
       end
@@ -114,7 +153,7 @@ module Excon
 
     def self.parse_headers(socket, datum)
       last_key = nil
-      until (data = socket.readline.chop!).empty?
+      until (data = socket.readline.chomp!).empty?
         if !data.lstrip!.nil?
           raise Excon::Errors::ResponseParseError, 'malformed header' unless last_key
           # append to last_key's last value
@@ -131,9 +170,10 @@ module Excon
 
     def initialize(params={})
       @data = {
-        :body     => '',
-        :headers  => Excon::Headers.new
+        :body     => ''
       }.merge(params)
+      @data[:headers] = Excon::Headers.new.merge!(params[:headers] || {})
+
       @body          = @data[:body]
       @headers       = @data[:headers]
       @status        = @data[:status]
@@ -151,6 +191,10 @@ module Excon
       data
     end
 
+    def pp
+      Excon::PrettyPrinter.pp($stdout, @data)
+    end
+
     # Retrieve a specific header value. Header names are treated case-insensitively.
     #   @param [String] name Header name
     def get_header(name)
