@@ -263,10 +263,11 @@ ensure
   end
 end
 
-def with_unicorn(name, file_name='/tmp/unicorn.sock')
+def with_unicorn(name, listen='127.0.0.1:9292')
   unless RUBY_PLATFORM == 'java'
     GC.disable if RUBY_VERSION < '1.9'
-    pid, w, r, e = Open4.popen4("unicorn", "-l", "unix://#{file_name}", rackup_path(name))
+    unix_socket = listen.sub('unix://', '') if listen.start_with? 'unix://'
+    pid, w, r, e = Open4.popen4("unicorn", "--no-default-middleware","-l", listen, rackup_path(name))
     until e.gets =~ /worker=0 ready/; end
   else
     # need to find suitable server for jruby
@@ -278,8 +279,8 @@ ensure
     GC.enable if RUBY_VERSION < '1.9'
     Process.wait(pid)
   end
-  if File.exist?(file_name)
-    File.delete(file_name)
+  if not unix_socket.nil? and File.exist?(unix_socket)
+    File.delete(unix_socket)
   end
 end
 
